/*	Public domain	*/

#include <agar/map/begin.h>
/* Begin generated block */
__BEGIN_DECLS
extern DECLSPEC void MAP_UpdateRefSel(MAP_View *_Nonnull, int,int);
__END_DECLS
/* Close generated block */
#include <agar/map/close.h>
