/* Agar icon reference file */
/* Generated by agarpaint 1.1 */

#include <agar/sg/begin.h>
/* Begin generated block */
__BEGIN_DECLS
extern DECLSPEC AG_StaticIcon sgIconLighting;
extern DECLSPEC AG_StaticIcon sgIconZBuffer;
extern DECLSPEC AG_StaticIcon sgIconWireframe;
extern DECLSPEC AG_StaticIcon sgIconVertices;
extern DECLSPEC AG_StaticIcon sgIconFacet;
extern DECLSPEC AG_StaticIcon sgIconFacetNormals;
extern DECLSPEC AG_StaticIcon sgIconVertexNormals;
extern DECLSPEC AG_StaticIcon sgIconCamera;
extern DECLSPEC AG_StaticIcon sgIconTriedre;
extern DECLSPEC AG_StaticIcon sgIconI;
extern DECLSPEC AG_StaticIcon sgIconJ;
extern DECLSPEC AG_StaticIcon sgIconK;
extern DECLSPEC AG_StaticIcon sgIconNode;
extern DECLSPEC AG_StaticIcon sgIconCgProgram;
extern DECLSPEC AG_StaticIcon sgIconOrigin;
extern DECLSPEC AG_StaticIcon sgIconLine;
extern DECLSPEC AG_StaticIcon sgIconTriangle;
extern DECLSPEC AG_StaticIcon sgIconCircle;
extern DECLSPEC AG_StaticIcon sgIconBezierCurve;
extern DECLSPEC AG_StaticIcon sgIconBlock;
extern DECLSPEC AG_StaticIcon sgIconText;
__END_DECLS
/* Close generated block */
#include <agar/sg/close.h>
