/*
 * Default Agar Stylesheet.
 *
 * Colors and font attributes of parents are inherited by default (unless
 * overridden by an instance Variable or matching stylesheet definition).
 * Padding and spacing attributes are not inherited by default.
 *
 * See: AG_StyleSheet(3) and "STYLE ATTRIBUTES" section of AG_Widget(3).
 */

AG_Window {
	padding: 0 3 4 3;                          /* top right bottom left */
	background-color: #575757;                 /* #RRGGBB */
}
AG_Box {
	padding: 2 3 2 3;
	spacing: 3 2;                              /* horiz vert */
}
AG_Titlebar {
	font-size: 90%;
	padding: 3;
	spacing: 1;
	color: rgb(40,50,60);
	color#disabled: rgb(35,35,35);
}
AG_Toolbar {
	padding: 0;
	spacing: 1;
}
AG_Textbox {
	spacing: 5;
}

AG_Checkbox {
	spacing: 5;
	background-color#hover: rgb(80,80,120);
}
AG_Console {
	font-family: courier-prime;
	padding: 4;
	background-color: rgb(0,0,0);
	background-color#hover: rgb(0,0,30);
	background-color#focused: rgb(5,5,5);
	background-color#disabled: rgb(10,10,10);
	text-color: rgb(240,240,240);
}
AG_Button {
	padding: 5 10 5 10;
	color#focused: #808081;
	color#hover: #818180;
	selection-color#focused: #646496;
	selection-color#hover: #969664;
}
AG_Editable {
	padding: 1;
	background-color: #0000;
}
AG_FileDlg {
	spacing: 2;
	padding: 3;
}
AG_Icon {
	spacing: 4 0;
}
AG_Label {
	padding: 2 6 2 6;
	background-color: #0000;
	background-color#focused: #0000;
}
AG_Menu {
	padding: 2 5 2 5;
	color: rgb(70,70,70);
	color#disabled: rgb(40,40,110);
	selection-color: rgb(40,40,110);
	text-color#disabled: rgb(170,170,170);
}
AG_MenuView {
	padding: 4 8 4 8;
	spacing: 8 0;
	color: rgb(70,70,70);
	selection-color: rgb(40,40,110);
	color#disabled: rgb(40,40,110);
	text-color#disabled: rgb(170,170,170);
}
AG_Pane {
	line-color#hover: rgb(200,200,180);
}
AG_Radio {
	spacing: 1;
	background-color#hover: rgb(80,80,120);
}
AG_ProgressBar {
	font-family: league-gothic;
	font-size: 90%;
	padding: 2;
}
AG_Scrollbar {
	line-color#hover: rgb(200,200,180);
}
AG_Separator {
	padding: 4;
}
AG_Statusbar {
	padding: 2;
	spacing: 1;
}
AG_Tlist {
	spacing: 4 0;
}
