/*	Public domain	*/

#include <agar/vg/begin.h>
/* Begin generated block */
__BEGIN_DECLS
extern DECLSPEC VG_ToolOps *_Nullable vgTools[];
extern DECLSPEC VG_ToolOps vgSelectTool;
extern DECLSPEC VG_ToolOps vgPointTool;
extern DECLSPEC VG_ToolOps vgLineTool;
extern DECLSPEC VG_ToolOps vgPolygonTool;
extern DECLSPEC VG_ToolOps vgCircleTool;
extern DECLSPEC VG_ToolOps vgArcTool;
extern DECLSPEC VG_ToolOps vgTextTool;
extern DECLSPEC VG_ToolOps vgProximityTool;
__END_DECLS
/* Close generated block */
#include <agar/vg/close.h>
