/*	Public domain	*/

/* Begin generated block */
__BEGIN_DECLS
extern DECLSPEC void M_BezierForm2(M_PointSet2*, M_PointSet2*);
extern DECLSPEC void M_BezierForm3(M_PointSet3* , M_PointSet3*);
extern DECLSPEC void M_BezierCurve2(M_PointSet2*, M_Real, M_Vector2*);
extern DECLSPEC void M_BezierCurve3(M_PointSet3*, M_Real, M_Vector3*);
__END_DECLS
/* Close generated block */
