/*	Public domain	*/

#ifndef _AGAR_MICRO_GUI_PUBLIC_H_
#define _AGAR_MICRO_GUI_PUBLIC_H_
#define _AGAR_MICRO_GUI_PUBLIC

#include <agar/core/core.h>

#include <agar/micro/gui.h>
#include <agar/micro/widget.h>
#include <agar/micro/window.h>
#include <agar/micro/primitive.h>

/*
#include <agar/micro/box.h>
...
*/

#endif /* _AGAR_MICRO_GUI_PUBLIC_H_ */
