/* Agar icon reference file */
/* Generated by agarpaint 1.1 */

#include <agar/map/begin.h>
/* Begin generated block */
__BEGIN_DECLS
extern DECLSPEC AG_StaticIcon mapIconStamp;
extern DECLSPEC AG_StaticIcon mapIconEraser;
extern DECLSPEC AG_StaticIcon mapIconNewView;
extern DECLSPEC AG_StaticIcon mapIconResize;
extern DECLSPEC AG_StaticIcon mapIconGrid;
extern DECLSPEC AG_StaticIcon mapIconLayerEditor;
extern DECLSPEC AG_StaticIcon mapIconSelectNode;
extern DECLSPEC AG_StaticIcon mapIconSelectItem;
extern DECLSPEC AG_StaticIcon mapIconMerge;
extern DECLSPEC AG_StaticIcon mapIconFill;
extern DECLSPEC AG_StaticIcon mapIconMirror;
extern DECLSPEC AG_StaticIcon mapIconFlip;
extern DECLSPEC AG_StaticIcon mapIconActor;
extern DECLSPEC AG_StaticIcon mapIconInvert;
extern DECLSPEC AG_StaticIcon mapIconSettings;
extern DECLSPEC AG_StaticIcon mapIconWalkable;
extern DECLSPEC AG_StaticIcon mapIconClimbable;
extern DECLSPEC AG_StaticIcon mapIconJumpable;
extern DECLSPEC AG_StaticIcon mapIconSlippery;
extern DECLSPEC AG_StaticIcon mapIconOrigin;
__END_DECLS
/* Close generated block */
#include <agar/map/close.h>
